;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_calib_pol
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION ibis_calib_pol, data, info_nb, tparams, xmtxl, xmtxr, filter, $
                         MASK = mask, INIT_MASK = init_mask, THRESH_MASK = thresh_mask, $
                         ROI = roi, WRANGE = wrange, $
                         SINGLE = single, DUAL = dual, $ 
                         RESIDUAL = residual, EXTERNALC = externalc, $ 
                         POLFAC = polfac

;;-------------------------------
;; Get some specifics 
;;-------------------------------

     modulation = ['I+Q', 'I+V', 'I-Q', 'I-V', 'I-U', 'I+U']
     nmod = n_elements(modulation)

     nlam = (size(data))[4] 

;;-------------------------------
;; Calculate T-Matrix
;;-------------------------------

     pc_azim   = avg(reform(info_nb.dst_az , nmod, nlam),1)
     pc_elev   = avg(reform(info_nb.dst_el , nmod, nlam),1)
     pc_table  = avg(reform(info_nb.dst_tbl, nmod, nlam),1)
     pc_pee    = avg(reform(info_nb.dst_pee, nmod, nlam),1)

     tp_azi    = mean( ( pc_azim ) )
     tp_elev   = mean( ( pc_elev ) )
     tp_table  = mean( ( pc_table ) )
     tp_pee    = mean( ( pc_pee ) )

;;-------------------------------
;; Calculate T-Matrix
;;-------------------------------

     tmtx  = ibis_tmtx(tparams, filter, tp_azi, tp_elev, tp_table)

;;-------------------------------
;; Demodulate 
;;-------------------------------

     PRINT,'Demodulating ...'

     ibis_demodulate, data, modulation, stokesin, DUAL = dual, SINGLE = single
     s = SIZE(stokesin)

;;-------------------------------
;; Calibrate for X and T
;;-------------------------------

     PRINT,'Calibration for X and T .....'

     ibis_calibrate_xt, stokesin, xmtxl, xmtxr, tmtx, stokesout, DUAL = dual, SINGLE = single

     stokesin = stokesout

;;-------------------------------
;; Calibrate for residual I->QUV
;;-------------------------------

     IF KEYWORD_SET(RESIDUAL) THEN BEGIN

        IF KEYWORD_SET(EXTERNALC) THEN BEGIN

           ;; in this case external results are used and the factors "polfac"
           ;; should be known.

           ibis_residual, stokesin, stokesout, $ 
                          MASK = mask, ROI = roi, WRANGE = wrange, $ 
                          POLFAC = polfac

        ENDIF ELSE BEGIN
            
              ;; in this case the data itself is used to determine the 
              ;; polarization factors for I->QUV via a masking procedure
              ;; or just defining a region. 
  
              PRINT,'Caclulation and calibration for residual I-> QUV using observations.....'

              IF KEYWORD_SET(MASK) THEN BEGIN

                 PRINT,'Using masking technique'

                 mask = ibis_i2quv_mask(stokesin, INIT_MASK = init_mask, THRESH_MASK = thresh_mask)

                 ibis_residual, stokesin, stokesout, $ 
                                MASK = mask, ROI = roi, WRANGE = wrange, $ 
                                POLFAC = polfac

              ENDIF

              IF KEYWORD_SET(ROI) THEN BEGIN

                 print,'Using simple ROI technique'

                 ibis_residual, stokesin, stokesout, $ 
                                MASK = mask, ROI = roi, WRANGE = wrange, $ 
                                POLFAC = polfac

              ENDIF

        ENDELSE

     ENDIF 

;-------------------------------
;; Rotate into correct RF on Sun
;-------------------------------

     PRINT,'Rotation into solar RF .....'

     DST_earth_latitude = 32.786 * !dtor
     r_frame_sphtri, DST_earth_latitude , tp_azi * !dtor, tp_elev * !dtor, $
                     siteha, sitedc, neg_parallactic
     parallactic = -neg_parallactic

     rotn = !pi + parallactic - tp_pee * !dtor
     crot = cos(-2.*rotn) 
     srot = sin(-2.*rotn)

     temp0 =  crot*reform(stokesout[*,*,*,1]) + srot*reform(stokesout[*,*,*,2])
     temp1 = -srot*reform(stokesout[*,*,*,1]) + crot*reform(stokesout[*,*,*,2])

     stokesout[*,*,*,1] = temp0
     stokesout[*,*,*,2] = temp1

;-------------------------------
;; Show result
;-------------------------------

;     window, 4, xs = 3*s[1], ys = s[2], title='Blue Wing'
;     tvscl, reform(stokesout[25:230,45:460,15,1]),0,0
;     tvscl, reform(stokesout[25:230,45:460,15,2]),s[1],0
;     tvscl, reform(stokesout[25:230,45:460,15,3]),2*s[1],0

;-------------------------------------------------
;; Done
;-------------------------------------------------

RETURN, stokesout

END
