;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_demodulate_polcal
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_demodulate_polcal, indata, modscheme, outdata, $
                            SINGLE = single, DUAL = dual

;--------------------------------
;; Get Modulation Scheme
;-------------------------------

     ipq = WHERE( modscheme EQ 'I+Q' )
     imq = WHERE( modscheme EQ 'I-Q' )
     ipv = WHERE( modscheme EQ 'I+V' )
     imv = WHERE( modscheme EQ 'I-V' )
     ipu = WHERE( modscheme EQ 'I+U' )
     imu = WHERE( modscheme EQ 'I-U' )

;-------------------------------------------------
; Differ between dual and single beam
;-------------------------------------------------

     IF KEYWORD_SET(DUAL) THEN BEGIN

        print,'---------------------------------------------------'
        print,'Ok, We are in Polarimetric dual-beam Mode                    '
        print,'---------------------------------------------------'

        LEFT = 0 & RIGHT = 1

        ; Create dual beam Stokes from temporal modulation 
        
        s = SIZE(indata)
        outdata = fltarr(s[1],4,2)

        itmp1q = REFORM( 0.5 * ( indata[*,ipq,LEFT] + indata[*,imq,LEFT] ) )
        itmp1u = REFORM( 0.5 * ( indata[*,ipu,LEFT] + indata[*,imu,LEFT] ) )
        itmp1v = REFORM( 0.5 * ( indata[*,ipv,LEFT] + indata[*,imv,LEFT] ) )

        itmp2q = REFORM( 0.5 * ( indata[*,ipq,RIGHT] + indata[*,imq,RIGHT] ) )
        itmp2u = REFORM( 0.5 * ( indata[*,ipq,RIGHT] + indata[*,imq,RIGHT] ) )
        itmp2v = REFORM( 0.5 * ( indata[*,ipq,RIGHT] + indata[*,imq,RIGHT] ) )

        outdata[*,0,LEFT] = ( itmp1q + itmp1u + itmp1v ) / 3.
        outdata[*,1,LEFT] = REFORM( 0.5 * ( indata[*,ipq,LEFT] - indata[*,imq,LEFT] ) )
        outdata[*,2,LEFT] = REFORM( 0.5 * ( indata[*,ipu,LEFT] - indata[*,imu,LEFT] ) )
        outdata[*,3,LEFT] = REFORM( 0.5 * ( indata[*,ipv,LEFT] - indata[*,imv,LEFT] ) )

        outdata[*,0,RIGHT] = ( itmp2q + itmp2u + itmp2v ) / 3.
        outdata[*,1,RIGHT] = REFORM( 0.5 * ( indata[*,imq,RIGHT] - indata[*,ipq,RIGHT] ) )
        outdata[*,2,RIGHT] = REFORM( 0.5 * ( indata[*,imu,RIGHT] - indata[*,ipu,RIGHT] ) )
        outdata[*,3,RIGHT] = REFORM( 0.5 * ( indata[*,imv,RIGHT] - indata[*,ipv,RIGHT] ) )

     ENDIF

     IF KEYWORD_SET(SINGLE) THEN BEGIN

        print,'---------------------------------------------------'
        print,'Ok, We are in Polarimetric  single-beam Mode                    '
        print,'---------------------------------------------------'

        s = SIZE(indata)
        outdata = fltarr(s[1],4)

        itmp1q = REFORM( 0.5 * ( indata[*,ipq] + indata[*,imq] ) )
        itmp1u = REFORM( 0.5 * ( indata[*,ipu] + indata[*,imu] ) )
        itmp1v = REFORM( 0.5 * ( indata[*,ipv] + indata[*,imv] ) )

        outdata[*,0] = ( itmp1q + itmp1u + itmp1v ) / 3.
        outdata[*,1] = REFORM( 0.5 * ( indata[*,ipq] - indata[*,imq] ) )
        outdata[*,2] = REFORM( 0.5 * ( indata[*,ipu] - indata[*,imu] ) )
        outdata[*,3] = REFORM( 0.5 * ( indata[*,ipv] - indata[*,imv] ) )

     ENDIF

;-------------------------------------------------
;; Done
;-------------------------------------------------

END
