;######################################################################
function pfunct, a0, X=x, Y=y, ERR=err

;; function: modeled polarization calibration curves
;; out of first guess Xmatrix elements and CU model.

     xmtx = setmtx(  a0[0],  a0[1],  a0[2],  a0[3]  $
                    ,a0[4],  a0[5],  a0[6],  a0[7]  $
                    ,a0[8],  a0[9],  a0[10], a0[11] $
                    ,a0[12], a0[13], a0[14], a0[15] )

     restore,'tmp_settings.sav'

     n = n_elements(cpolang)

;;----------------------------------
;; calculate CU Mueller matrices
;;----------------------------------

     radeg   = !pi/180.

     umtx = setmtx(   1.0000,   0.0000,   0.0000,    0.0000  $
                    , 0.0000,   1.0000,   0.0000,    0.0000  $
                    , 0.0000,   0.0000,   1.0000,    0.0000  $
                    , 0.0000,   0.0000,   0.0000,    1.0000  )

;; the linear polarizer

     px = 1
     py = 0. ;;; 0.005
     a = ( px^2 + py^2 )
     b = ( px^2 - py^2 )
     d  = 2. * px * py 

     polmtx = setmtx(  a,   b,   0.,  0.  $
                     , b,   a,   0.,  0.  $
                     , 0.,  0.,  d,   0.  $
                     , 0.,  0.,  0.,  d   )

     polmtx = polmtx * 0.5 

     polmtx = transpose(polmtx)

;; the retarder

;;     bb = 1.

     px = 1 + a0[16]/2.
     py = 1 - a0[16]/2.

     sq = sqrt(px*py)
     a  = ( px + py )/2.
     c  = ( px - py )/2.

     sd = sin( a0[17] * radeg )
     cd = cos( a0[17] * radeg  )

     retmtx = setmtx(  a,    c,     0.,     0.   $
                     , c,    a,     0.,     0.   $
                     , 0.,  0.,  sq*cd,  -sq*sd  $
                     , 0.,  0.,  sq*sd,   sq*cd  )

     retmtx = transpose(retmtx)

     stksin = fltarr(n,4)

     for j = 0,n-1 do begin

         if tflag eq 1 then begin

            tmtx  = ibis_tmtx(tparams.values, (tparams.lambda)[0], $ 
                              (tparams.azi[j])[0], (tparams.elev[j])[0], (tparams.table[j])[0] )

         endif 

         if  tflag eq 0 then tmtx = umtx

         tmtx = tmtx / tmtx[0,0]

         vecin = transpose(tmtx ## [1.,0.,0.,0.])

         ;;----------------------------------
         ;; calculate CU Mueller matrices
         ;;----------------------------------

         ;;----------------------------------
         ;; linear polarizer is in the beam
         ;;----------------------------------

         if icalp[j] eq 1 then begin

            ;; rotation of polarizer

            c = cos( 2. * cpolang[j] * radeg )
            s = sin( 2. * cpolang[j] * radeg )

            rot1 = setmtx(   1,   0.,   0.,  0.  $
                            , 0.,   c,   s,   0.  $
                            , 0.,  -s,   c,   0.  $
                            , 0.,   0.,  0.,  1   )

            rot1 = transpose(rot1)

            rot2 = setmtx(  1,   0.,   0.,  0.  $
                           , 0.,   c,  -s,   0.  $
                           , 0.,   s,   c,   0.  $
                           , 0.,   0.,  0.,  1   )
 
            rot2 = transpose(rot2)

            ;; rotation to fast axis

            vecout = rot1 ## vecin

            ;; apply polarizer

            vecout = polmtx ## vecout

            ;; rotate back 

            vecout = rot2 ## vecout

            ;;----------------------------------
            ;; retarder is in the beam
            ;;----------------------------------

            if icalr[j] eq 1 then begin

               ;; rotation of retarder

               c = cos( 2. * (cretang[j]+a0[18]) * radeg )
               s = sin( 2. * (cretang[j]+a0[18]) * radeg )

               rot1 = setmtx(  1,   0.,   0.,  0.  $
                           , 0.,   c,   s,   0.  $
                           , 0.,  -s,   c,   0.  $
                           , 0.,   0.,  0.,  1   )

               rot1 = transpose(rot1)


               rot2 = setmtx(  1,   0.,   0.,  0.  $
                           , 0.,   c,  -s,   0.  $
                           , 0.,   s,   c,   0.  $
                           , 0.,   0.,  0.,  1   )
 
               rot2 = transpose(rot2)

               ;; rotation to fast axis

               vecout = rot1 ## vecout

               ;; apply retarder

               vecout = retmtx ## vecout

               ;; rotate to input frame

               vecout = rot2 ## vecout

            endif

         endif

         if icalp[j] eq 0 then begin

            if icalr[j] eq 1 then begin

               ;; rotation of retarder

               c = cos( 2. * (cretang[j] + a0[18]) * radeg )
               s = sin( 2. * (cretang[j] + a0[18]) * radeg )

               rot1 = setmtx(  1,   0.,   0.,  0.  $
                             , 0.,   c,   s,   0.  $
                             , 0.,  -s,   c,   0.  $
                             , 0.,   0.,  0.,  1   )

               rot1 = transpose(rot1)


               rot2 = setmtx(  1,   0.,   0.,  0.  $
                             , 0.,   c,  -s,   0.  $
                             , 0.,   s,   c,   0.  $
                             , 0.,   0.,  0.,  1   )
 
               rot2 = transpose(rot2)

               ;; rotation to fast axis

               vecout = rot1 ## vecin

               ;; apply retarder

               vecout = retmtx ## vecout

               ;; rotate to input frame

               vecout = rot2 ## vecout

            endif

            if icalr[j] eq 0 then begin

               print,'There is something wrong, neither retarder nor polarizer are in the beam!'
               stop

            endif

         endif

      ;;---------------------------------
      ;; 6. normalize
      ;;---------------------------------

          stks         = vecout ;;;; / vecout[0] 
          stksin[j, *] = REFORM( stks )

     endfor

     ff = xmtx ## stksin

      ;;---------------------------------------------------------
      ;; fit for polarization fraction; 
      ;; do not fit for intensity
      ;;---------------------------------------------------------

     model = fltarr((size(x))[1])
     model[0:(size(x))[1]/4-1] = ff[*,0]/ff[*,0] 
     model[(size(x))[1]/4:2*(size(x))[1]/4-1] = ff[*,1]/ff[*,0] 
     model[2*(size(x))[1]/4:3*(size(x))[1]/4-1] = ff[*,2]/ff[*,0] 
     model[3*(size(x))[1]/4:4*(size(x))[1]/4-1] = ff[*,3]/ff[*,0] 

      ;;---------------------------------------------------------
      ;; fit for polarization fraction; 
      ;; do fit for intensity
      ;;---------------------------------------------------------

;;     model = fltarr((size(x))[1])
;;     model[0:(size(x))[1]/4-1] = ff[*,0]
;;     model[(size(x))[1]/4:2*(size(x))[1]/4-1] = ff[*,1]/ff[*,0] 
;;     model[2*(size(x))[1]/4:3*(size(x))[1]/4-1] = ff[*,2]/ff[*,0] 
;;     model[3*(size(x))[1]/4:4*(size(x))[1]/4-1] = ff[*,3]/ff[*,0] 


     save, model, filename = 'tmp_model.sav' 

  return, (y-model)/err
end
;######################################################################
function pfunct1, a0, X=x, Y=y, ERR=err

;; function: modeled polarization calibration curves
;; out of first guess Xmatrix elements and CU model.

     xmtx = setmtx(  a0[0],  a0[1],  a0[2],  a0[3]  $
                    ,a0[4],  a0[5],  a0[6],  a0[7]  $
                    ,a0[8],  a0[9],  a0[10], a0[11] $
                    ,a0[12], a0[13], a0[14], a0[15] )

     restore,'tmp_settings.sav'

     restore,'tmp_cal.idl'

     n = n_elements(cpolang)

;;----------------------------------
;; calculate CU Mueller matrices
;;----------------------------------

     radeg   = !pi/180.

     umtx = setmtx(   1.0000,   0.0000,   0.0000,    0.0000  $
                    , 0.0000,   1.0000,   0.0000,    0.0000  $
                    , 0.0000,   0.0000,   1.0000,    0.0000  $
                    , 0.0000,   0.0000,   0.0000,    1.0000  )

;; the linear polarizer

     px = 1
     py = 0. ;;; 0.005
     a = ( px^2 + py^2 )
     b = ( px^2 - py^2 )
     d  = 2. * px * py 

     polmtx = setmtx(  a,   b,   0.,  0.  $
                     , b,   a,   0.,  0.  $
                     , 0.,  0.,  d,   0.  $
                     , 0.,  0.,  0.,  d   )

     polmtx = polmtx * 0.5 

     polmtx = transpose(polmtx)

;; the retarder

;;     bb = 1.

     px = 1 + dichroic/2.
     py = 1 - dichroic/2.

     sq = sqrt(px*py)
     a  = ( px + py )/2.
     c  = ( px - py )/2.

     sd = sin( ret * radeg )
     cd = cos( ret * radeg  )

     retmtx = setmtx(  a,    c,     0.,     0.   $
                     , c,    a,     0.,     0.   $
                     , 0.,  0.,  sq*cd,  -sq*sd  $
                     , 0.,  0.,  sq*sd,   sq*cd  )

     retmtx = transpose(retmtx)

     stksin = fltarr(n,4)

     for j = 0,n-1 do begin

         if tflag eq 1 then begin

            tmtx  = ibis_tmtx(tparams.values, (tparams.lambda)[0], $ 
                              (tparams.azi[j])[0], (tparams.elev[j])[0], (tparams.table[j])[0] )

         endif else tmtx = umtx

         vecin = transpose(tmtx ## [1.,0,0,0])

         ;;----------------------------------
         ;; calculate CU Mueller matrices
         ;;----------------------------------

         ;;----------------------------------
         ;; linear polarizer is in the beam
         ;;----------------------------------

         if icalp[j] eq 1 then begin

            ;; rotation of polarizer

             c = cos( 2. * cpolang[j] * radeg )
             s = sin( 2. * cpolang[j] * radeg )

             rot1 = setmtx(   1,   0.,   0.,  0.  $
                            , 0.,   c,   s,   0.  $
                            , 0.,  -s,   c,   0.  $
                            , 0.,   0.,  0.,  1   )

             rot1 = transpose(rot1)

             rot2 = setmtx(  1,   0.,   0.,  0.  $
                           , 0.,   c,  -s,   0.  $
                           , 0.,   s,   c,   0.  $
                           , 0.,   0.,  0.,  1   )
 
             rot2 = transpose(rot2)

            ;; rotation to fast axis

             vecout = rot1 ## vecin

            ;; apply polarizer

             vecout = polmtx ## vecout

            ;; rotate back 

             vecout = rot2 ## vecout

            ;;----------------------------------
            ;; retarder is in the beam
            ;;----------------------------------

            if icalr[j] eq 1 then begin

               ;; rotation of retarder

                   c = cos( 2. * (cretang[j]+alpha) * radeg )
                   s = sin( 2. * (cretang[j]+alpha) * radeg )

                   rot1 = setmtx(  1,   0.,   0.,  0.  $
                           , 0.,   c,   s,   0.  $
                           , 0.,  -s,   c,   0.  $
                           , 0.,   0.,  0.,  1   )

                   rot1 = transpose(rot1)


                   rot2 = setmtx(  1,   0.,   0.,  0.  $
                           , 0.,   c,  -s,   0.  $
                           , 0.,   s,   c,   0.  $
                           , 0.,   0.,  0.,  1   )
 
                   rot2 = transpose(rot2)

               ;; rotation to fast axis

                   vecout = rot1 ## vecout

               ;; apply retarder

                   vecout = retmtx ## vecout

              ;; rotate to input frame

                   vecout = rot2 ## vecout

            endif

         endif

         if icalp[j] eq 0 then begin

            if icalr[j] eq 1 then begin

               ;; rotation of retarder

               c = cos( 2. * (cretang[j] + alpha) * radeg )
               s = sin( 2. * (cretang[j] + alpha) * radeg )

               rot1 = setmtx(  1,   0.,   0.,  0.  $
                             , 0.,   c,   s,   0.  $
                             , 0.,  -s,   c,   0.  $
                             , 0.,   0.,  0.,  1   )

               rot1 = transpose(rot1)


               rot2 = setmtx(  1,   0.,   0.,  0.  $
                             , 0.,   c,  -s,   0.  $
                             , 0.,   s,   c,   0.  $
                             , 0.,   0.,  0.,  1   )
 
               rot2 = transpose(rot2)

               ;; rotation to fast axis

               vecout = rot1 ## vecin

               ;; apply retarder

               vecout = retmtx ## vecout

               ;; rotate to input frame

               vecout = rot2 ## vecout

            endif

            if icalr[j] eq 0 then begin

               print,'There is something wrong, neither retarder nor polarizer are in the beam!'
               stop

            endif

         endif

      ;;---------------------------------
      ;; 6. normalize
      ;;---------------------------------

          stks         = vecout ; / vecout[0] 
          stksin[j, *] = REFORM( stks )

     endfor

     ff = xmtx ## stksin

      ;;---------------------------------------------------------
      ;; fit for polarization fraction; 
      ;; do not fit for intensity
      ;;---------------------------------------------------------

     model = fltarr((size(x))[1])
     model[0:(size(x))[1]/4-1] = ff[*,0]/ff[*,0] 
     model[(size(x))[1]/4:2*(size(x))[1]/4-1] = ff[*,1]/ff[*,0] 
     model[2*(size(x))[1]/4:3*(size(x))[1]/4-1] = ff[*,2]/ff[*,0] 
     model[3*(size(x))[1]/4:4*(size(x))[1]/4-1] = ff[*,3]/ff[*,0] 


      ;;---------------------------------------------------------
      ;; fit for polarization fraction; 
      ;; do fit for intensity
      ;;---------------------------------------------------------

;;     model = fltarr((size(x))[1])
;;     model[0:(size(x))[1]/4-1] = ff[*,0]
;;     model[(size(x))[1]/4:2*(size(x))[1]/4-1] = ff[*,1]/ff[*,0] 
;;     model[2*(size(x))[1]/4:3*(size(x))[1]/4-1] = ff[*,2]/ff[*,0] 
;;     model[3*(size(x))[1]/4:4*(size(x))[1]/4-1] = ff[*,3]/ff[*,0] 

     save, model, filename = 'tmp_model.sav' ;;;;;; ,/verb

  return, (y-model)/err
end
;######################################################################
  pro fit_polcal, aa, fcnargs, parinfo, yproffit 

         yproffit = mpfit('pfunct', aa, FUNCTARGS = fcnargs, PARINFO = parinfo, /QUIET, /autoderiv)

  end
;######################################################################
  pro fit_polcal1, aa, fcnargs, parinfo, yproffit 

         yproffit = mpfit('pfunct1', aa, FUNCTARGS = fcnargs, PARINFO = parinfo, /QUIET, /autoderiv)

  end
;######################################################################
pro ibis_xmatrix, infile, outpath, lambda, date, $
                      TMAT = tmat

     radeg   = !pi/180.
     phi = 70. ;; phi
     theta_off = 0. ;;
     bb = 1.

     ntable = 4
     npoints = 28

     restore,infile, /verb

     if keyword_set(TMAT) then begin

        index = where(in_beam_retarder eq 1 or in_beam_lin_pol eq 1)
        icalr = in_beam_retarder[index]
        icalp = in_beam_lin_pol[index]
        cretang = float(cal_settings_retarder[index])
        cpolang = float(cal_settings_lin_pol[index])

        Ntotal = n_elements(index)

        RESTORE, tmat, /verb ;;; tt

        tparams = create_struct('values', dblarr(n_elements(tt)), $
                                'azi', dblarr(Ntotal), $ 
                                'lambda', float(1), $ 
                                'elev', dblarr(Ntotal), $
                                'table', dblarr(Ntotal)) 

        tparams.values = tt
        tparams.lambda = lambda
        tparams.azi = telparam_azi[index]
        tparams.elev = telparam_elev[index]
        tparams.table = telparam_table[index]

        tflag = 1
        save, icalr, icalp, cretang, cpolang, tflag, tparams, $
              filename = 'tmp_settings.sav'

     endif else begin

           index = where(in_beam_retarder eq 1 and in_beam_lin_pol eq 1)
           icalr = in_beam_retarder[index]
           icalp = in_beam_lin_pol[index]
           cretang = float(cal_settings_retarder[index])
           cpolang = float(cal_settings_lin_pol[index])

           tflag = 0
           save, icalr, icalp, cretang, cpolang, tflag, $
                 filename = 'tmp_settings.sav'

     endelse

     s = SIZE(stokesvect)
 
;;-------------------------------------------------
;; Rearrange arrays
;;-------------------------------------------------

     in_beam_clear = intarr(s[1])
     table_angle   = fltarr(s[1])

     for i = 0,ntable-1 do begin

         in_beam_clear[i*npoints] = 1
         in_beam_clear[(i+1)*npoints-1] = 1
         table_angle[i*npoints:(i+1)*npoints-1] = telparam_table[i]

     endfor

     index_clear = where(in_beam_clear eq 1)
     index_dark = where(in_beam_dark_slide eq 1)

;;--------------------------------------------------
;; do both beams
;;--------------------------------------------------

     for ibeam = 0, 1 do begin

         beam = ibeam ;;; which beam? a or b : 0 or 1
     
         if beam eq 0 then outfile = '01.ta.' +date+ '.' +lambda+ '.X.idl'
         if beam eq 1 then outfile = '01.tb.' +date+ '.' +lambda+ '.X.idl'

         ;;--------------------------------------------------
         ;; intensity normalization with clear information
         ;; and dark subtraction for Stokes I
         ;;--------------------------------------------------

         icleara = REFORM( stokesvect[index_clear, 0, beam] ) 
         idarka = REFORM( stokesvect[index_dark, 0, beam] ) 

         ic1a = avg( [icleara[0],icleara[1]] )
         ic2a = avg( [icleara[2],icleara[3]] )
         ic3a = avg( [icleara[4],icleara[5]] )
         ic4a = avg( [icleara[6],icleara[7]] )
         id1a = avg( [idarka[0],idarka[1]] )
         id2a = avg( [idarka[2],idarka[3]] )
         id3a = avg( [idarka[4],idarka[5]] )
         id4a = avg( [idarka[6],idarka[7]] )

         a1 = replicate(ic1a,n_elements(index))
         a2 = replicate(ic2a,n_elements(index))
         a3 = replicate(ic3a,n_elements(index))
         a4 = replicate(ic4a,n_elements(index))
         b1 = replicate(id1a,n_elements(index))
         b2 = replicate(id2a,n_elements(index))
         b3 = replicate(id3a,n_elements(index))
         b4 = replicate(id4a,n_elements(index))

         iffma = [ a1, a2, a3, a4]
         idcma = [ b1, b2, b3, b4]

         im1 = ( stokesvect[index,0,beam] - idcma)/(iffma-idcma)
         im2 = stokesvect[index,1,beam]/(iffma-idcma)
         im3 = stokesvect[index,2,beam]/(iffma-idcma)
         im4 = stokesvect[index,3,beam]/(iffma-idcma)

         ;;---------------------------------------------------------
         ;; fit for polarization fraction; 
         ;; do not fit for intensity
         ;;---------------------------------------------------------

         stksout = [ [fltarr((size(im1))[1])+1d],[im2/im1],[im3/im1],[im4/im1] ]

         ;;----------------------------------------------------
         ;; fit for polarization fraction; 
         ;; do fit for intensity
         ;;----------------------------------------------------

;;;         stksout = [ [im1],[im2/im1],[im3/im1],[im4/im1] ]

         ;;-----------------------------------------------
         ;; Initial guess of curve and xmatrix
         ;;-----------------------------------------------

         if keyword_set(TMAT) then begin

            stksin = construct_initial_guess_new(phi, bb, theta_off, icalr, icalp, cpolang, cretang, $
                                                 TPARAMS = tparams)


            matrix_inversion,transpose(stksin), transpose(stksout), xmat, error

            print,' First guess'
            print,xmat/xmat[0,0]

         endif else begin

               stksin = construct_initial_guess_new(phi, bb, theta_off, icalr, icalp, cpolang, cretang)

               matrix_inversion,transpose(stksin), transpose(stksout), xmat, error

               print,' First guess'
               print,xmat/xmat[0,0]

         endelse

         ;;------------------------------------------------
         ;; iniatial values for the fitting from first guess
         ;;------------------------------------------------

         ind = indgen(16)
         aa = fltarr(19)
         aa[0:15] = xmat[ind]
         aa[16] = bb           ;; dichroism
         aa[17] = phi          ;; retardance of retarder
         aa[18] = theta_off    ;; offset angle of retarder

         nn = n_elements(index) 
         h = fltarr(nn*ntable)
         h[0:nn-1] = stksout[0:nn-1,0]
         h[nn:2*nn-1] = stksout[0:nn-1,1]
         h[2*nn:3*nn-1] = stksout[0:nn-1,2]
         h[3*nn:4*nn-1] = stksout[0:nn-1,3]

         xh = findgen(nn*ntable)

         parinfo = replicate({value:0.D, fixed:0, limited:[0,0], $
                             limits:[0.D,0]}, 19)
         ;  parinfo(0).fixed = 1
         ;  parinfo(1).limited(0) = 1
         ;  parinfo(1).limited(1) = 1
         ;  parinfo(1).limits(0)  = a1
         ;  parinfo(1).limits(1)  = 2.D
         ;  parinfo(*).value = aa

        x = xh
        y = h
        err = fltarr(n_elements(y))+1.

        fcnargs = {X:x, Y:y, Err:err}

        fit_polcal, aa, fcnargs, parinfo, aa_fit

         xmtx_fit = setmtx(  aa_fit[0],  aa_fit[1],  aa_fit[2],  aa_fit[3]  $
                            ,aa_fit[4],  aa_fit[5],  aa_fit[6],  aa_fit[7]  $
                            ,aa_fit[8],  aa_fit[9],  aa_fit[10], aa_fit[11] $
                            ,aa_fit[12], aa_fit[13], aa_fit[14], aa_fit[15] )

         ;;------------------------------------------------
         ;; whats the fit????
         ;;------------------------------------------------

         print,' Result'
         print,xmtx_fit/xmtx_fit[0,0]
         print,aa_fit[16:*]

         ;;------------------------------------------------
         ;; Calculate residuals
         ;;------------------------------------------------

         dichroic = aa_fit[16]
         ret = aa_fit[17]
         alpha = aa_fit[18]

         save,dichroic,ret,alpha,filename='tmp_cal.idl'

         restore,'tmp_model.sav'

         nn = n_elements(index) ; 64
         stksin = stksout
         stksin[*,0] = model[0:nn-1]
         stksin[*,1] = model[nn:2*nn-1]
         stksin[*,2] = model[2*nn:3*nn-1] 
         stksin[*,3] = model[3*nn:4*nn-1]

         matrix_inversion, transpose(stksin), transpose(stksout), xmat_resid, error_resid 

         print,' Residuals'
         print,xmat_resid/xmat_resid[0,0]

         ;;------------------------------------------------
         ;; Save data for that beam
         ;;------------------------------------------------

         print, outpath + outfile

         save, xmtx_fit, xmat, xmat_resid, h, model, error, error_resid, dichroic, ret, alpha, $
               filename = outpath + outfile


     endfor

;;------------------------------------------------
;; Done
;;------------------------------------------------


end
