;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_aperture
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2006-Nov, Han Uitenbroek, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION ibis_aperture, frame, DRADIUS=dradius, PLOT=plot

;; Determine the IBIS aperture in frame by determining the extreme
;; locations where the maxima in the derivatives of the frame in x and y
;; appear. When /PLOT is set verify result by plotting results.

  aperture = create_struct('x0', 0.0, 'y0', 0.0, 'radius', 0.)

  IF (NOT keyword_set(DRADIUS)) THEN dradius = 0.0

  dim = size(frame)
  Nx  = dim[1]
  Ny  = dim[2]

  ;; First the extreme locations in x.

  xl = Nx-1
  xr = 0
  FOR j=Ny/4, 3*Ny/4 DO BEGIN
    xdiam = frame[*, j]
    xddx = 0.5 * (xdiam - shift(xdiam, 1) + shift(xdiam, -1) - xdiam)
    dxmax = max(xddx, x1, SUBSCRIPT_MIN=x2)
    xl = min([xl, x1])
    xr = max([xr, x2])
  ENDFOR

  ;; Then the extreme locations in y.

  yb = Ny-1
  yt = 0
  FOR i=Nx/4, 3*Nx/4 DO BEGIN
    ydiam = reform(frame[i, *])
    yddy = 0.5 * (ydiam - shift(ydiam, 1) + shift(ydiam, -1) - ydiam)
    dymax = max(yddy, y1, SUBSCRIPT_MIN=y2)
    yb = min([yb, y1])
    yt = max([yt, y2])
  ENDFOR

  ;; Store results in aperture structure.

  aperture.x0 = 0.5*(xr + xl)
  aperture.y0 = 0.5*(yb + yt)
  aperture.radius = 0.25 * (xr - xl + yt - yb) + dradius

  ;; graph results if requested.

  IF (keyword_set(PLOT)) THEN BEGIN
    tvscl, frame
    oplot_ibis_aperture, aperture, COLOR=255B
  ENDIF

  return, aperture
END
