;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_file_structure
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       tmp = ibis_file_structure(file, filter, INFO = info)
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       tmp = structure 
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION ibis_file_structure, file, filter, INFO = info

;;-------------------------------------------
;; Number of accumulations if modulated
;;-------------------------------------------

nacc = 1.

;;-------------------------------------------
;; Read the header of extended fits file
;;-------------------------------------------

info0 = ibis_header(file)

;;-------------------------------------------
;; Read the header of extended fits file
;;-------------------------------------------

index   = WHERE(info0.filter EQ filter)
nfilter = n_elements(index)
lambda  = info0.grid[index]

nlambda       = n_elements(unique_elements(lambda))
lambda_unique = unique_elements(lambda)

stokes        = info0.stokes[index]
nstokes       = n_elements(unique_elements(stokes))
stokes_unique = unique_elements(stokes)

extension = info0.extension[index]

;;-------------------------------------------
;; Output
;;-------------------------------------------

nroption_i            = intarr(nlambda)
nroption_full         = intarr(nlambda)
nroption_transverse   = intarr(nlambda)
nroption_longitudinal = intarr(nlambda)
nrepeat_i             = intarr(1)
nrepeat_full          = intarr(1)
nrepeat_transverse    = intarr(1)
nrepeat_longitudinal  = intarr(1)

;;-------------------------------------------
;; Indices : 6 defined modulation states
;;-------------------------------------------

index_stokes_i0    = where(stokes eq 'I')
index_stokes_no_i0 = where(stokes ne 'I')
index_stokes_i1    = where(stokes eq 'I+Q')
index_stokes_i2    = where(stokes eq 'I+V')
index_stokes_i3    = where(stokes eq 'I-Q')
index_stokes_i4    = where(stokes eq 'I-V')
index_stokes_i5    = where(stokes eq 'I-U')
index_stokes_i6    = where(stokes eq 'I+U')

;;-------------------------------------------
;; Criteria
;;-------------------------------------------

crit_i0 = index_stokes_i0[0] eq -1
crit_i1 = index_stokes_i1[0] eq -1
crit_i2 = index_stokes_i2[0] eq -1
crit_i3 = index_stokes_i3[0] eq -1
crit_i4 = index_stokes_i4[0] eq -1
crit_i5 = index_stokes_i5[0] eq -1
crit_i6 = index_stokes_i6[0] eq -1

no_crit_i0 = index_stokes_i0[0] ne -1
no_crit_i1 = index_stokes_i1[0] ne -1
no_crit_i2 = index_stokes_i2[0] ne -1
no_crit_i3 = index_stokes_i3[0] ne -1
no_crit_i4 = index_stokes_i4[0] ne -1
no_crit_i5 = index_stokes_i5[0] ne -1
no_crit_i6 = index_stokes_i6[0] ne -1

;;***************************************
;; Criteria that identify PURE PROGRAMS
;;***************************************

;; only spectroscopy

crit_spectroscopy = crit_i1 and crit_i2 and crit_i3 and crit_i4 and crit_i5 and crit_i6

;; only polarimetry 

crit_longitudinal = crit_i0 and crit_i1 and crit_i3 and crit_i5 and crit_i6 and no_crit_i2 and no_crit_i4
crit_transverse   = crit_i0 and crit_i2 and crit_i4 and no_crit_i1 and no_crit_i3 and no_crit_i5 and no_crit_i6
crit_full         = crit_i0 and no_crit_i1 and no_crit_i2 and no_crit_i3 and no_crit_i4 $
                            and no_crit_i5 and no_crit_i6
crit_polarimetry  = crit_longitudinal or crit_transverse or crit_full

;;***************************************
;; Criteria that identify MIXED PROGRAMS
;;***************************************

;; polarimetry mixed with spectroscopy

crit_longitudinal_mixed_i = no_crit_i0 and crit_i1 and crit_i3 and crit_i5 and crit_i6 and no_crit_i2 and no_crit_i4
crit_transverse_mixed_i   = no_crit_i0 and crit_i2 and crit_i4 and no_crit_i1 and no_crit_i3 and no_crit_i5 and no_crit_i6
crit_full_mixed_i         = no_crit_i0 and no_crit_i1 and no_crit_i2 and no_crit_i3 and no_crit_i4 $
                                     and no_crit_i5 and no_crit_i6
crit_mixed_i = crit_longitudinal_mixed_i or crit_transverse_mixed_i or crit_full_mixed_i

;; polarimetry mixed with polarimetry ;;;; so ein Quatsch ....

;crit_longitudinal_mixed = no_crit_i0 and crit_i1 and crit_i3 and crit_i5 and crit_i6 and no_crit_i2 and no_crit_i4
;crit_transverse_mixed   = no_crit_i0 and crit_i2 and crit_i4 and no_crit_i1 and no_crit_i3 and no_crit_i5 and no_crit_i6
;crit_full_mixed         = no_crit_i0 and no_crit_i1 and no_crit_i2 and no_crit_i3 and no_crit_i4 $
;                                     and no_crit_i5 and no_crit_i6
;crit_mixed = crit_longitudinal_mixed or crit_transverse_mixed or crit_full_mixed

;;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;;
;; MAIN PART : DIFFER BEWTEEN PROGRAMS
;;
;;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

;;-----------------------------------------------------
;; Spectroscopy onlywith or without repeats and R > 1
;;-----------------------------------------------------

if crit_spectroscopy then begin  

   ;;; now we have to differ between repeats and the R-option
   
   if nfilter eq nlambda then begin

      nrepeat_i[*]  = 1
      nroption_i[*] = 1
      extension_spectroscopy = extension

   endif

   if nfilter gt nlambda then begin

      for i = 0, nlambda-1 do begin

          index_lambda    = where(lambda eq lambda_unique[i])
          nindex          = n_elements(index_lambda)
          extension_index = extension[index_lambda]

          tmp = (extension_index - shift(extension_index,1))[1:*]

          index_tmp = where(tmp ne 1)

          if index_tmp[0] eq -1 then begin

             nrepeat_i[*]  = 1
             nroption_i[i] = n_elements(tmp) + 1

          endif else begin

                    nrepeat_i[*]  = n_elements(index_tmp) + 1
                    nroption_i[i] = n_elements(tmp[0:index_tmp[0]-1]) + 1

                endelse
      endfor

      if nrepeat_i[0] eq 1 then extension_spectroscopy = extension
      if nrepeat_i[0] gt 1 then extension_spectroscopy = extension ; [0:total(nroption_i)-1]

   endif


endif

;;-----------------------------------------------------
;; Polarimetry only with or without repeats and R > 1
;;-----------------------------------------------------

if crit_polarimetry then begin 

   if nfilter eq nstokes*nlambda then begin

      if crit_full then begin

         nrepeat_full[*] = 1
         nroption_full[*] = 1
         extension_full = extension
;;;         nmod_full = nstokes

      endif

      if crit_transverse then begin

         nrepeat_transverse[*] = 1
         nroption_transverse[*] = 1
         extension_transverse = extension

      endif

      if crit_longitudinal then begin

         nrepeat_longitudinal[*] = 1
         nroption_longitudinal[*] = 1
         extension_longitudinal = extension

      endif


   endif

   if nfilter gt nstokes*nlambda then begin

      for i = 0, nlambda-1 do begin

          index_lambda = where(lambda eq lambda_unique[i])
          nindex = n_elements(index_lambda) / nacc
          extension_index = extension[index_lambda]

          tmp = (extension_index - shift(extension_index,1))[1:*]

          index_tmp = where(tmp ne 1)

          if index_tmp[0] eq -1 then begin

             if crit_full then begin

                nrepeat_full[*] = 1
                nroption_full[i] = ( n_elements(tmp) + 1 ) / ( nstokes * nacc)

             endif

             if crit_transverse then begin

                nrepeat_transverse[*] = 1
                nroption_transverse[i] = ( n_elements(tmp) + 1 ) / ( nstokes * nacc)

             endif

             if crit_longitudinal then begin

                nrepeat_longitudinal[*] = 1
                nroption_longitudinal[i] = ( n_elements(tmp) + 1 ) / ( nstokes * nacc)

             endif

          endif else begin

                     if crit_full then begin

                        nrepeat_full[*] = n_elements(index_tmp) + 1
                        nroption_full[i] = ( n_elements(tmp[0:index_tmp[0]-1]) + 1 ) / ( nstokes * nacc)

                     endif

                     if crit_longitudinal then begin

                        nrepeat_longitudinal[*] = n_elements(index_tmp) + 1
                        nroption_longitudinal[i] = ( n_elements(tmp[0:index_tmp[0]-1]) + 1 ) / ( nstokes * nacc)

                     endif

                     if crit_transverse then begin

                        nrepeat_transverse[*] = n_elements(index_tmp) + 1
                        nroption_transverse[i] = ( n_elements(tmp[0:index_tmp[0]-1]) + 1 ) / ( nstokes * nacc)

                     endif

                endelse

      endfor

      if nrepeat_full[0] eq 1 then extension_full = extension
      if nrepeat_transverse[0] eq 1 then extension_transverse = extension
      if nrepeat_longitudinal[0] eq 1 then extension_longitudinal = extension

      if nrepeat_full[0] gt 1 then extension_full = extension ;[0:total(nroption_full)-1]
      if nrepeat_longitudinal[0] gt 1 then extension_longitudinal = extension ;[0:total(nroption_longitudinal)-1]
      if nrepeat_transverse[0] gt 1 then extension_transverse = extension ;[0:total(nroption_transverse)-1]

   endif

endif

;;------------------------------------------------------------------------
;; Polarimetry and spectroscopy with or without repeats and R > 1 in each
;;------------------------------------------------------------------------

if crit_mixed_i then begin 

   index_i = index_stokes_i0
   nfilter_i = n_elements(index_i)
   lambda_i = lambda[index_i]

   nlambda_i = n_elements(unique_elements(lambda_i))
   lambda_unique_i = unique_elements(lambda_i)

   stokes_i = stokes[index_i]
   nstokes_i = n_elements(unique_elements(stokes_i))
   stokes_unique_i = unique_elements(stokes_i)

   extension_i = extension[index_i]

   ;;; now we have to differ between repeats and the R-option
   
   if nfilter_i eq nlambda_i then begin

      nrepeat_i = 1
      nroption_i[*] = 1
      extension_spectroscopy = extension_i

   endif

   if nfilter_i gt nlambda_i then begin

      for i = 0, nlambda_i-1 do begin

          index_lambda_i = where(lambda_i eq lambda_unique_i[i])
          nindex_i = n_elements(index_lambda_i)
          extension_index_i = extension_i[index_lambda_i]

          tmp = (extension_index_i - shift(extension_index_i,1))[1:*]

          index_tmp = where(tmp ne 1)

          if index_tmp[0] eq -1 then begin

             nrepeat_i = 1
             nroption_i[i] = n_elements(tmp) + 1

          endif else begin

                    nrepeat_i = n_elements(index_tmp) + 1
                    nroption_i[i] = n_elements(tmp[0:index_tmp[0]-1]) + 1

                endelse
      endfor

      if nrepeat_i[0] eq 1 then extension_spectroscopy = extension_i
      if nrepeat_i[0] gt 1 then extension_spectroscopy = extension_i ; [0:total(nroption_i)-1]

   endif

   if crit_full_mixed_i then begin 

      index_full = index_stokes_no_i0
      nfilter_full = n_elements(index_full)
      lambda_full = lambda[index_full]

      nlambda_full = n_elements(unique_elements(lambda_full))
      lambda_unique_full = unique_elements(lambda_full)

      stokes_full = stokes[index_full]
      nstokes_full = n_elements(unique_elements(stokes_full))
      stokes_unique_full = unique_elements(stokes_full)

      extension_mixed_full = extension[index_full]

      if nfilter_full eq nstokes_full*nlambda_full then begin

         nrepeat_full[*] = 1
         nroption_full[*] = 1
         extension_full = extension_mixed_full

      endif

      if nfilter_full gt nstokes_full*nlambda_full then begin

         for i = 0, nlambda_full-1 do begin

             index_lambda_full = where(lambda_full eq lambda_unique_full[i])
             nindex_full = n_elements(index_lambda_full) / nacc
             extension_index_full = extension_mixed_full[index_lambda_full]

             tmp = (extension_index_full - shift(extension_index_full,1))[1:*]

             index_tmp = where(tmp ne 1)

             if index_tmp[0] eq -1 then begin

                nrepeat_full[*] = 1
                nroption_full[i] = ( n_elements(tmp) + 1 ) / ( nstokes_full * nacc)

             endif else begin

                        nrepeat_full[*]  = n_elements(index_tmp) + 1
                        nroption_full[i] = ( n_elements(tmp[0:index_tmp[0]-1]) + 1 ) / ( nstokes_full * nacc)

                   endelse 

         endfor

         if nrepeat_full[0] eq 1 then extension_full = extension_mixed_full
         if nrepeat_full[0] gt 1 then extension_full = extension_mixed_full ;[0:total(nroption_full)-1]

      endif

   endif

   if crit_longitudinal_mixed_i then begin 

      index_longitudinal = index_stokes_no_i0
      nfilter_longitudinal = n_elements(index_longitudinal)
      lambda_longitudinal = lambda[index_longitudinal]

      nlambda_longitudinal = n_elements(unique_elements(lambda_longitudinal))
      lambda_unique_longitudinal = unique_elements(lambda_longitudinal)

      stokes_longitudinal = stokes[index_longitudinal]
      nstokes_longitudinal = n_elements(unique_elements(stokes_longitudinal))
      stokes_unique_longitudinal = unique_elements(stokes_longitudinal)

      extension_mixed_longitudinal = extension[index_longitudinal]

      if nfilter_longitudinal eq nstokes_longitudinal*nlambda_longitudinal then begin

         nrepeat_longitudinal[*] = 1
         nroption_longitudinal[*] = 1
         extension_longitudinal = extension_mixed_longitudinal

      endif

      if nfilter_longitudinal gt nstokes_longitudinal*nlambda_longitudinal then begin

         for i = 0, nlambda_longitudinal-1 do begin

             index_lambda_longitudinal = where(lambda_longitudinal eq lambda_unique_longitudinal[i])
             nindex_longitudinal = n_elements(index_lambda_longitudinal) / nacc
             extension_index_longitudinal = extension_mixed_longitudinal[index_lambda_longitudinal]

             tmp = (extension_index_longitudinal - shift(extension_index_longitudinal,1))[1:*]

             index_tmp = where(tmp ne 1)

             if index_tmp[0] eq -1 then begin

                nrepeat_longitudinal[*] = 1
                nroption_longitudinal[i] = ( n_elements(tmp) + 1 ) / ( nstokes_longitudinal * nacc)

             endif else begin

                        nrepeat_longitudinal[*]  = n_elements(index_tmp) + 1
                        nroption_longitudinal[i] = ( n_elements(tmp[0:index_tmp[0]-1]) + 1 ) / ( nstokes_longitudinal * nacc)

                   endelse 

         endfor

         if nrepeat_longitudinal[0] eq 1 then extension_longitudinal = extension_mixed_longitudinal
         if nrepeat_longitudinal[0] gt 1 then extension_longitudinal = extension_mixed_longitudinal ;[0:total(nroption_longitudinal)-1]

      endif

   endif

   if crit_transverse_mixed_i then begin 

      index_transverse = index_stokes_no_i0
      nfilter_transverse = n_elements(index_transverse)
      lambda_transverse = lambda[index_transverse]

      nlambda_transverse = n_elements(unique_elements(lambda_transverse))
      lambda_unique_transverse = unique_elements(lambda_transverse)

      stokes_transverse = stokes[index_transverse]
      nstokes_transverse = n_elements(unique_elements(stokes_transverse))
      stokes_unique_transverse = unique_elements(stokes_transverse)

      extension_mixed_transverse = extension[index_transverse]

      if nfilter_transverse eq nstokes_transverse*nlambda_transverse then begin

         nrepeat_transverse[*] = 1
         nroption_transverse[*] = 1

      endif

      if nfilter_transverse gt nstokes_transverse*nlambda_transverse then begin

         for i = 0, nlambda_transverse-1 do begin

             index_lambda_transverse = where(lambda_transverse eq lambda_unique_transverse[i])
             nindex_transverse = n_elements(index_lambda_transverse) / nacc
             extension_index_transverse = extension_mixed_transverse[index_lambda_transverse]

             tmp = (extension_index_transverse - shift(extension_index_transverse,1))[1:*]

             index_tmp = where(tmp ne 1)

             if index_tmp[0] eq -1 then begin

                nrepeat_transverse[*] = 1
                nroption_transverse[i] = ( n_elements(tmp) + 1 ) / ( nstokes_transverse * nacc)

             endif else begin

                        nrepeat_transverse[*]  = n_elements(index_tmp) + 1
                        nroption_transverse[i] = ( n_elements(tmp[0:index_tmp[0]-1]) + 1 ) / ( nstokes_transverse * nacc)

                   endelse 

         endfor

         if nrepeat_transverse[0] eq 1 then extension_transverse = extension_mixed_transverse
         if nrepeat_transverse[0] gt 1 then extension_transverse = extension_mixed_transverse ; [0:total(nroption_transverse)-1]

      endif

   endif

endif

;;------------------------------------------------------------------------
;; Generate output structures
;;------------------------------------------------------------------------

if crit_spectroscopy then begin

   extension_numbers = create_struct('spectroscopy', intarr(n_elements(extension_spectroscopy)))

   extension_numbers.spectroscopy = extension_spectroscopy

endif

if crit_full then begin

   extension_numbers = create_struct('full', intarr(n_elements(extension_full)))

   extension_numbers.full = extension_full

endif

if crit_transverse then begin

   extension_numbers = create_struct('transverse', intarr(n_elements(extension_transverse)))

   extension_numbers.transverse = extension_transverse

endif


if crit_longitudinal then begin

   extension_numbers = create_struct('longitudinal', intarr(n_elements(extension_longitudinal)))

   extension_numbers.longitudinal = extension_longitudinal

endif


if crit_full_mixed_i then begin

   extension_numbers = create_struct('spectroscopy', intarr(n_elements(extension_spectroscopy)), $
                                     'full', intarr(n_elements(extension_full)))
   extension_numbers.spectroscopy = extension_spectroscopy
   extension_numbers.full = extension_full

endif

if crit_transverse_mixed_i then begin

   extension_numbers = create_struct('spectroscopy', intarr(n_elements(extension_spectroscopy)), $
                                     'transverse', intarr(n_elements(extension_transverse)))
   extension_numbers.spectroscopy = extension_spectroscopy
   extension_numbers.transverse = extension_transverse

endif


if crit_longitudinal_mixed_i then begin

   extension_numbers = create_struct('spectroscopy', intarr(n_elements(extension_spectroscopy)), $
                                     'longitudinal', intarr(n_elements(extension_longitudinal)))
   extension_numbers.spectroscopy = extension_spectroscopy
   extension_numbers.longitudinal = extension_longitudinal

endif


info = create_struct('crit_spectroscopy', 0, $
                    'crit_full', 0, $
                    'crit_transverse', 0, $
                    'crit_longitudinal', 0, $
                    'crit_polarimetry', 0, $
                    'crit_full_mixed_i', 0, $
                    'crit_transverse_mixed_i', 0, $
                    'crit_longitudinal_mixed_i', 0, $
                    'crit_mixed_i', 0, $
                    'nroption_i', intarr(nlambda), $ 
                    'nroption_full', intarr(nlambda), $
                    'nroption_transverse', intarr(nlambda), $ 
                    'nroption_longitudinal', intarr(nlambda), $ 
                    'filter', intarr(1), $
                    'nrepeat_i', intarr(1), $ 
                    'nrepeat_full', intarr(1), $ 
                    'nrepeat_transverse', intarr(1), $ 
                    'nrepeat_longitudinal', intarr(1))

info.crit_spectroscopy = crit_spectroscopy
info.crit_full = crit_full
info.crit_transverse = crit_transverse
info.crit_longitudinal = crit_longitudinal
info.crit_polarimetry = crit_polarimetry
info.crit_full_mixed_i = crit_full_mixed_i
info.crit_transverse_mixed_i = crit_transverse_mixed_i
info.crit_longitudinal_mixed_i = crit_longitudinal_mixed_i
info.crit_mixed_i = crit_mixed_i
info.nroption_i = nroption_i 
info.nroption_full = nroption_full
info.nroption_transverse = nroption_transverse
info.nroption_longitudinal = nroption_longitudinal
info.filter = filter
info.nrepeat_i = nrepeat_i
info.nrepeat_full = nrepeat_full
info.nrepeat_transverse = nrepeat_transverse
info.nrepeat_longitudinal = nrepeat_longitudinal

;;------------------------------------------------------------------------
;; Done
;;------------------------------------------------------------------------
                   
return, extension_numbers
end
