;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_find_images
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Jun, Kevin Reardon, INAF/Queens University Belfast
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION ibis_find_images, input_dir, F_NAME=F_NAME, COUNT=COUNT, DST_INFO=DST_INFO

IF N_ELEMENTS(f_name) LE 0 THEN f_name = '*fits*'

num_input_dirs  = N_ELEMENTS(input_dir)
fits_files      = FILE_SEARCH(input_dir[0], f_name, $
                      COUNT=num_fits_files, /FOLD_CASE, /EXPAND_TILDE)
IF num_input_dirs GE 1 THEN BEGIN
    FOR dir=1,num_input_dirs-1 DO BEGIN
        fits_files_more      = FILE_SEARCH(input_dir[dir], f_name, $
                                   COUNT=num_fits_files_more, /FOLD_CASE, /EXPAND_TILDE)
        fits_files           = [fits_files, fits_files_more]
        num_fits_files       = num_fits_files + num_fits_files_more
    ENDFOR
ENDIF
PRINT,num_fits_files

skip_header     = 0
;PRINT,skip_header NE 1 

max_num_ims     = (num_fits_files * 5000L) < 3e5

image_file      = STRARR(max_num_ims)
image_exten     = INTARR(max_num_ims)
image_date      = STRARR(max_num_ims)
image_keywords  = ['NAXIS1', 'NAXIS2', 'BINNING', 'EXPTIME', $
                   'FILTER', 'WAVELNTH', 'REL_WAVE', 'WAVEUNIT', $
                   'FP1_VOLT', 'FP2_VOLT', 'LCVR1_RE', 'LCVR2_RE', 'STOKES', $
                   'DELTA_T', 'FRAMENUM' ]
dst_keywords    = ['DST_TIME', 'DST_AZ', 'DST_EL', 'DST_TBL', 'DST_PEE', $
                   'DST_BEE0', 'DST_SEE', 'DST_SLAT', 'DST_SLNG', 'DST_LLVL']
num_im_keywords   = N_ELEMENTS(image_keywords)
num_dst_keywords  = N_ELEMENTS(dst_keywords)
image_hdr_info    = STRARR(max_num_ims, num_im_keywords)
image_hdr_dst     = STRARR(max_num_ims, num_dst_keywords)
detector_name     = ''
instrument_name   = ''
cnt               = 0

IF num_fits_files GE 1 THEN BEGIN

    FOR filenum = 0,num_fits_files-1 DO BEGIN
      FITS_OPEN, fits_files[filenum], fcb
      num_exten = fcb.NEXTEND

      IF num_exten EQ 0 THEN BEGIN
          FITS_READ, fcb, data, main_header, /HEADER_ONLY
          image_file[cnt]  = fits_files[filenum]
          image_exten[cnt] = 0
          image_date[cnt]  = SXPAR(main_header, 'DATE-OBS')
          instrument_name  = SXPAR(main_header, 'INSTRUME')
          
          sxpar_detector = STRTRIM(SXPAR(main_header, 'DETECTOR'),2)
          sxpar_camera   = STRTRIM(SXPAR(main_header, 'CAMERA'),2)
          IF STRLEN(sxpar_camera)   GE 10 THEN detector_name = sxpar_camera
          IF STRLEN(sxpar_detector) GE 10 THEN detector_name = sxpar_detector

          FOR kywrd = 0,num_im_keywords-1 DO BEGIN
              image_hdr_info[cnt, kywrd] = SXPAR(main_header, image_keywords[kywrd])
          ENDFOR
          
          FOR kywrd = 0,num_dst_keywords-1 DO BEGIN
              image_hdr_dst[cnt, kywrd]  = SXPAR(main_header, dst_keywords[kywrd])
          ENDFOR
 
          cnt += 1
      ENDIF ELSE BEGIN
          FITS_READ, fcb, data, main_header, /HEADER_ONLY, EXTEN=0
          instrument_name  = SXPAR(main_header, 'INSTRUME')

          sxpar_detector = STRTRIM(SXPAR(main_header, 'DETECTOR'),2)
          sxpar_camera   = STRTRIM(SXPAR(main_header, 'CAMERA'),2)
          IF STRLEN(sxpar_camera)   GE 10 THEN detector_name = sxpar_camera
          IF STRLEN(sxpar_detector) GE 10 THEN detector_name = sxpar_detector
          
          FOR extnum = 1,num_exten DO BEGIN
            FITS_READ, fcb, data, main_header, EXTEN=extnum, /HEADER_ONLY
            image_file[cnt]  = fits_files[filenum]
            image_exten[cnt] = extnum
            image_date[cnt]  = SXPAR(main_header, 'DATE-OBS')

            FOR kywrd = 0,num_im_keywords-1 DO BEGIN
                image_hdr_info[cnt, kywrd] = SXPAR(main_header, image_keywords[kywrd])
            ENDFOR

            FOR kywrd = 0,num_dst_keywords-1 DO BEGIN
                image_hdr_dst[cnt, kywrd]  = SXPAR(main_header, dst_keywords[kywrd])
            ENDFOR
            
            cnt += 1
          ENDFOR
          
      ENDELSE
       
      FITS_CLOSE, fcb
    ENDFOR

    ;PRINT,cnt
    count = cnt

    image_file     = image_file[0:cnt-1]
    image_exten    = image_exten[0:cnt-1]
    image_date     = image_date[0:cnt-1]
    image_hdr_info = image_hdr_info[0:cnt-1, *]
    image_hdr_dst  = image_hdr_dst[0:cnt-1, *]
    image_wave     = FLOAT(image_hdr_info[*, WHERE(STRMATCH(image_keywords,'WAVELNTH'))])
    image_wavunit  = FLOAT(image_hdr_info[*, WHERE(STRMATCH(image_keywords,'WAVEUNIT'))])
    image_wave_ang = image_wave * 10^image_wavunit * 1e10
    
    images_info = CREATE_STRUCT('instrument',    instrument_name, $
                                'detector',      detector_name, $
                                'image_file',    image_file,$
                                'image_exten',   image_exten, $
                                'image_date',    image_date, $
                                'naxis1',        image_hdr_info[*,WHERE(STRMATCH(image_keywords,'NAXIS1'))], $
                                'naxis2',        image_hdr_info[*,WHERE(STRMATCH(image_keywords,'NAXIS2'))], $
                                'binning',       image_hdr_info[*,WHERE(STRMATCH(image_keywords,'BINNING'))], $
                                'exptime',       image_hdr_info[*,WHERE(STRMATCH(image_keywords,'EXPTIME'))], $
                                'filter',        image_hdr_info[*,WHERE(STRMATCH(image_keywords,'FILTER'))], $
                                'wavelength',    image_wave_ang, $
                                'rel_wave',      image_hdr_info[*,WHERE(STRMATCH(image_keywords,'REL_WAVE'))], $
                                'voltage_fp1',   image_hdr_info[*,WHERE(STRMATCH(image_keywords,'FP1_VOLT'))], $
                                'voltage_fp2',   image_hdr_info[*,WHERE(STRMATCH(image_keywords,'FP2_VOLT'))], $
                                'ret_lcvr1',     image_hdr_info[*,WHERE(STRMATCH(image_keywords,'LCVR1_RE'))], $
                                'ret_lcvr2',     image_hdr_info[*,WHERE(STRMATCH(image_keywords,'LCVR2_RE'))], $
                                'stokes',        image_hdr_info[*,WHERE(STRMATCH(image_keywords,'STOKES'))], $
                                'delta_t',       image_hdr_info[*,WHERE(STRMATCH(image_keywords,'DELTA_T'))], $
                                'framenum',      image_hdr_info[*,WHERE(STRMATCH(image_keywords,'FRAMENUM'))] )
  
                                                              
    dst_info =    CREATE_STRUCT('image_file',    image_file,$
                                'image_exten',   image_exten, $
                                'image_date',    image_date, $
                                'dst_time',      image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_TIME'))], $
                                'azim',          image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_AZ'))], $
                                'elev',          image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_EL'))], $
                                'table',         image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_TBL'))], $
                                'p_angle',       image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_PEE'))], $
                                'b_angle',       image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_BEE0'))], $
                                'see',           image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_SEE'))], $
                                'slat',          image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_SLAT'))], $
                                'slng',          image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_SLNG'))], $
                                'llevel',        image_hdr_dst[*,WHERE(STRMATCH(dst_keywords,'DST_LLVL'))] )
 

ENDIF
    
RETURN,images_info

END
