;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       avg_dark_nb
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       dark = avg_dark_nb(ddir, nstart, nend, dlevel, drms, info_dark)
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       dark = averaged dark array
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION avg_dark_nb, ddir, nstart, nend, dlevel, drms, info_dark

     dfiles = file_search( ddir + '*.DarkCalibration.fits', COUNT = nd)
     ni = nend - nstart + 1 

     sz = ibis_size(dfiles[0]) 

     dark = FLTARR(sz[0], sz[1], ni)
     nimages = lonarr(nd)

     FOR k = 0, nd-1 DO nimages[k] = ibis_nimages(dfiles[k])

     ntotal = total(nimages)

     PRINT,'------------------------------------------------------'
     PRINT,'Dark files found: ', nd
     PRINT,'Number of dark images in individual files: ', nimages
     PRINT,'Total number of dark images: ', ntotal
     PRINT,'Total number of dark images for this filter: ', nd*ni
     PRINT,'------------------------------------------------------'

     PRINT & PAUSE,'Enter files not used for average dark (def = use all, enter 1 2 3 4 5 ...)', /nopause, ans = ans

     IF ans EQ '' THEN BEGIN

	PRINT,'Processing all'
       
        n = nd

     ENDIF ELSE BEGIN

           ans = fix(get_words(ans))
           n = nd - n_elements(ans)
           index = ibis_good_frame_index(nd,ans)
           dfiles = dfiles[index]
           nimages = nimages[index]
           ntotal = total(nimages)
   
     ENDELSE
         
     dlevel = FLTARR(ni*nd)
     drms   = FLTARR(ni*nd)
     
     l = 0

     info_dark = ibis_create_struct(ni*nd)

     FOR k = 0, n-1 DO BEGIN

         PRINT,'Dark file : ', dfiles[k]

         FITS_OPEN, dfiles[k], tmp

         FOR i = 1L, ni DO BEGIN
 
             FITS_READ, tmp, data, header, exten_no= nstart+i-1, /NO_PDU
             info_dark = ibis_info(header, info_dark, dfiles[k], nstart+i-1, l)
             statist,FLOAT(data)
             dlevel[l] = AVG(FLOAT(data))
             drms[l] = stdev(data)/avg(data)
             dark[*,*,i-1] += FLOAT(data)
             l = l+1

         ENDFOR

         FITS_CLOSE, tmp

         window,0
         plot, dlevel, psym=-4, xtitle = 'image number', ytitle = 'mean dark counts'

     ENDFOR     

     window,0
     plot, info_dark.time-info_dark.time[0], dlevel, psym=-4, xtitle = 'time   [sec]', ytitle = 'mean dark counts'
     window,1
     plot, info_dark.time-info_dark.time[0], drms, psym=-4, xtitle = 'time   [sec]', ytitle = 'rms'

     dark /= FLOAT(n)

     PRINT,'----------------------------'
     PRINT,'statistics mean dark :      '
     PRINT,'----------------------------'

     statist,dark

     RETURN, dark
END
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       avg_flat_nb
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       flat = avg_flat_nb(fdir, dark, nstart, nend, flevel, frms, info_flat)
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       flat =     averaged flat array
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION avg_flat_nb, fdir, dark, nstart, nend, flevel, frms, info_flat 

     ffiles = file_search( fdir + '*.FlatFieldCalibration.fits', COUNT = nd)
     ni = nend - nstart + 1 

     sz = ibis_size(ffiles[0]) 

     flat = FLTARR(sz[0], sz[1], ni)
     nimages = lonarr(nd)

     FOR k = 0, nd-1 DO begin
         print, ffiles[k]
         nimages[k] = ibis_nimages(ffiles[k])
     endfor

     ntotal = total(nimages)

     PRINT,'------------------------------------------------------'
     PRINT,'Flat files found: ', nd
     PRINT,'Number of flat images in individual files: '
     FOR k = 0, nd-1 DO print, nimages[k]
     PRINT,'Total number of flat images: ', ntotal
     PRINT,'Total number of flat images for this filter: ', nd*ni
     PRINT,'------------------------------------------------------'

     fflevel = fltarr(ni,nd)

     PRINT,'------------------------------------------------------'
     PRINT,'Calculating FOV averaged signal in all images ........'
     PRINT,'------------------------------------------------------'

;;;;     fflevel = ibis_quick_look(ffiles)

     FOR k = 0, nd-1 DO BEGIN

         print,'Flat file : ', ffiles[k]

         FITS_OPEN, ffiles[k], tmp

         FOR i = 1L, ni DO BEGIN

             FITS_READ, tmp, data, header, exten_no= nstart+i-1, /NO_PDU
             fflevel[i-1,k] = AVG(FLOAT(data))

         ENDFOR

         FITS_CLOSE, tmp

     ENDFOR     
       
     tmp = fflevel / rebin(median(fflevel, dim = 2), ni, nd)
     mask = tmp GE 0.9

     window, 0, xsize = 700, ysize=400, title='Average flat signal in all scans'
     plot, avg(fflevel,0), psym=-4,xtitle = 'scan number', ytitle = 'average counts in scan'

     window, 2, xsize = ni*5, ysize=nd*10, title='Average flat signal in individual images'
     tvscl, SCALE(tmp, 5, 10,/sample)>0.8
     winplot
     xyouts, 10, 10, 'spectral information removed', color=0,charsize=1.5,charthick=2

     window, 4, xsize = ni*5, ysize=nd*10, title='0.9 % Mask'
     tvscl, SCALE(mask, 5, 10,/sample)>0.8
     winplot
     for i=0,10*nd-1,10 do xyouts, 1,i, string(i/10), color=0,charsize=1,charthick=2

     PRINT & PAUSE,'Do you want to process all scans for the flatfield averaging process (yes/no)?', /nopause, ans = ans

     IF ans EQ 'yes' THEN BEGIN

	PRINT,'Processing all'
       
        n = nd
        mask = intarr(ni,nd) + 1.
        ntotal = ni*nd

     ENDIF

     IF ans EQ 'no' THEN BEGIN

        PRINT & PAUSE,'Do you want to exclude complete scans for the averaging (yes) process or salvage images from cloudy scans (oje)', /nopause, ans = ans1

        IF ans1 EQ 'yes' THEN BEGIN

           PRINT & PAUSE,'Enter scans not used for average flat (enter 0 1 2 3 ...)', /nopause, ans = ans2

           ans2 = fix(get_words(ans2))
           n = nd - n_elements(ans2)
           index = ibis_good_frame_index(nd,ans2)
           ffiles = ffiles[index]
           nimages = nimages[index]
           ntotal = ni * n_elements(nimages) ;;total(nimages)
 
           mask = intarr(ni,n) + 1. 

        ENDIF 

        IF ans1 EQ 'oje' THEN BEGIN

           n = nd 
           mask = tmp GE 0.9
           ntotal = total(mask)

        ENDIF 

     ENDIF
         
     wdelete
     wdelete

     flevel  = FLTARR(ntotal)
     frms    = FLTARR(ntotal)

     l = 0
       
     PRINT,'------------------------------------------------------'
     PRINT,'Total number of flat images used: ', ntotal
     PRINT,'------------------------------------------------------'

     info_flat = ibis_create_struct(ntotal)

     FOR k = 0, n-1 DO BEGIN

         print,'Flat file : ', ffiles[k]
         FITS_OPEN, ffiles[k], tmp

         FOR i = 1L, ni DO BEGIN

             IF mask[i-1,k] EQ 1 THEN BEGIN

                FITS_READ, tmp, data, header, exten_no = nstart+i-1, /NO_PDU
                info_flat = ibis_info(header, info_flat, ffiles[k], nstart+i-1, l)
                flevel[l] = AVG(FLOAT(data))
                frms[l] = stdev(data)/avg(data)
                flat[*,*,i-1] += FLOAT(data)
                l = l+1
             
             ENDIF ELSE BEGIN

                PRINT,'Flat image not used .... '

             ENDELSE

         ENDFOR

         FITS_CLOSE, tmp

         window,0,title='FOV averaged flat counts'
         plot, flevel, psym=-4, xtitle = 'image number', ytitle = 'mean flat counts'

     ENDFOR     
         
     window,0,title='FOV averaged flat signal'
     plot, info_flat.time-info_flat.time[0], flevel, psym=-4, xtitle = 'time   [sec]', ytitle = 'mean flat counts'
     window,1,title='RMS of flat signal'
     plot, info_flat.time-info_flat.time[0], frms, psym=-4, xtitle = 'time   [sec]', ytitle = 'rms'
     window,2,title='DST guider lightlevel'
     plot, info_flat.time-info_flat.time[0], info_flat.dst_llevel, psym=-4, xtitle = 'time   [sec]', ytitle = 'DST guider light level'

     FOR i = 0, ni-1 DO flat[*,*,i] = reform(flat[*,*,i]) / total(mask[i,*])

;;     flat /= FLOAT(nd)

     PRINT,'----------------------------'
     PRINT,'statistics mean flat :      '
     PRINT,'----------------------------'

     statist,flat

     RETURN, flat
END
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_flat_dark_avg_nb
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       ibis_flat_dark_avg_nb, flat_dir, dark_dir, offile, odfile, n1, n2
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_flat_dark_avg_nb, flat_dir, dark_dir, offile, odfile, n1, n2

;;***************************************************************
;; create mean dark if necessary
;;***************************************************************

     IF (FILE_TEST(odfile)) THEN BEGIN
 
        PRINT,'----------------------------------'
        PRINT, 'Mean dark file already exists    '
        PRINT,'----------------------------------'

        RESTORE, odfile, /verb

     ENDIF ELSE BEGIN

           dark = avg_dark_nb(dark_dir, n1, n2, dark_level, dark_rms, info_dark_nb)

           PRINT,'------------------------------'
           PRINT,'save dark: ', odfile
           PRINT,'------------------------------'

           SAVE, dark, dark_level, dark_rms, info_dark_nb, FILENAME = odfile

     ENDELSE

     PRINT,''

;;***************************************************************
;; create mean flat if necessary
;;***************************************************************

     IF (FILE_TEST(offile)) THEN BEGIN

        PRINT,'-----------------------------------'
        PRINT, 'Mean flat file already exists  !! '
        PRINT,'-----------------------------------'

     ENDIF ELSE BEGIN

           flat = avg_flat_nb(flat_dir, dark, n1, n2, flat_level, flat_rms, info_flat_nb)

           PRINT,'------------------------------'
           PRINT,'save flat: ', offile
           PRINT,'------------------------------'

           SAVE, flat, flat_level, flat_rms, info_flat_nb, FILENAME = offile
 
     ENDELSE

     PRINT,''

;;***************************************************************
;; Done!
;;***************************************************************

END
