;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_prepare
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       ibis_prepare, sdir, wdir, osdir, owdir, wldfile, wlffile, $ 
;                     nstart, nend, afile, filter_cwl, $ 
;                     ORIENTATION = orientation, NREPEAT = nrepeat
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_prepare, sdir, wdir, osdir, owdir, wldfile, wlffile, nstart, nend, afile, filter_cwl, $ 
                  ORIENTATION = orientation, NREPEAT = nrepeat

;;***************************************************************
;; Data
;;***************************************************************

     sfiles = file_search( sdir + '*.ScienceObservation.fits', COUNT = nnb)
     ni = nend - nstart + 1 
     sz = read_size(sfiles[0])

     wfiles = file_search( wdir + '*.ScienceObservation.fits', COUNT = nbb)
     sz = read_size(wfiles[0])

     PRINT,'---------------------------------------'
     PRINT,'Science narrowband files found: ', nnb
     PRINT,'Science broadband files found: ', nbb
     PRINT,'---------------------------------------'

;;***************************************************************
;; Load aux files
;;***************************************************************

     RESTORE, /VERB, wldfile
     RESTORE, /VERB, wlffile
     RESTORE, /VERB, afile

     r   = parameter.r
     f1  = parameter.f1
     f2  = parameter.f2
     sfl = parameter.sfl

;;***************************************************************
;; Create mask for gain
;;***************************************************************

     msk = create_aperture(flat, 256, 64, 0.65, 11)
     mask = msk
     brush = [[0,1,0],[1,1,1],[0,1,0]]
     FOR i = 0,10 DO mask = ERODE(mask, brush)   
     index_msk = WHERE(mask EQ 1, COMPLEMENT = index0)
     gain  = flat / AVG( flat[index_msk] )

;;***************************************************************
;; Loop through data
;;***************************************************************

     l = 0

     FOR n = 0, nnb-1 DO BEGIN

         print,'Narrowband file : ', sfiles[n]
         print,'Broadband file : ', wfiles[n]

         FITS_OPEN, sfiles[n], tmp_nb
         FITS_OPEN, wfiles[n], tmp_bb

     FOR k = 0, nrepeat - 1 DO BEGIN

         nb_data = FLTARR(sz[0], sz[1], ni)
         bb_data = FLTARR(sz[0], sz[1], ni)

         info_bb = ibis_create_struct(ni)
         info_nb = ibis_create_struct(ni)

         FOR i = 1L, ni DO BEGIN

             FITS_READ, tmp_nb, data, header, exten_no = nstart+(k*ni)+i-1, /NO_PDU
             nb_data[*,*,i-1] = data
             info_nb = ibis_info(header, info_nb, sfiles[n], nstart+(k*ni)+i-1, i-1)

             FITS_READ, tmp_bb, data, header, exten_no = nstart+(k*ni)+i-1, /NO_PDU
             data = ( data - dark ) / gain

             IF orientation EQ 'default' THEN data = REVERSE(ROTATE(data,1))

             data = congrid(data, f1*sz[0], f2*sz[1], cubic=-0.5)
             data = rot(data,r,cubic=-0.5)
             datan = ibis_set(data,sz[0],sz[1])
             datan = shift_bicub(datan,sfl[0],sfl[1])

             bb_data[*,*,i-1] = datan
             info_bb = ibis_info(header, info_bb, sfiles[n], nstart+(k*ni)+i-1, i-1)

         ENDFOR

         ;;***************************************************************
         ;; Save data
         ;;***************************************************************

         ;;
         ;; Save narrowband and broadband separately.
         ;; exposure times and other information is included.
         ;;

         filename1 = osdir + $
                           string(filter_cwl, l, FORMAT='(I4, "_nb", I3.3, ".sav")')
         filename2 = owdir + $
                           string(filter_cwl, l, FORMAT='(I4, "_bb", I3.3, ".sav")')

         PRINT,'-------------------------------------'
         PRINT,'Scan saved : ', filename1
         PRINT,'Whitelight saved : ', filename2
         PRINT,'-------------------------------------'
                    
         SAVE, FILENAME = filename1, nb_data, info_nb

         SAVE, FILENAME = filename2, bb_data, info_bb

         l = l+1

     ENDFOR

         FITS_CLOSE, tmp_nb
         FITS_CLOSE, tmp_bb

     ENDFOR

END

