;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_quick_look.pro
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       tmp = FOV averaged signal in all files
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION ibis_quick_look, dir, fname, FILTER = filter

     IF KEYWORD_SET(FILTER) THEN BEGIN

        files = file_search( dir + fname, COUNT = nfiles)

        nimages = lonarr(nfiles)

        FOR k = 0, nfiles-1 DO BEGIN

            info_header = ibis_header(files[k])

            index = WHERE(info_header.filter EQ filter)

            IF index[0] eq -1 THEN BEGIN

               nimages[k] = 0

               PRINT,'------------------------------------------------------'
               PRINT,'No filter position found in file  ', files[k]
               PRINT,'------------------------------------------------------'
  
            ENDIF ELSE BEGIN

                  nimages[k] = n_elements(index)

            ENDELSE

        ENDFOR

        ntotal = total(nimages)

        IF ntotal EQ 0 THEN BEGIN

           PRINT,'This filter position was not used !'
           RETURN,0

        ENDIF

        PRINT,'------------------------------------------------------'
        PRINT,'Files found: ', nfiles
        PRINT,'Number of images in individual files: '
        FOR k = 0, nfiles-1 DO print, nimages[k]
        PRINT,'Total number of images: ', ntotal
        PRINT,'------------------------------------------------------'

        llevel = fltarr(max(nimages),nfiles)

        PRINT,'------------------------------------------------------'
        PRINT,'Calculating FOV averaged signal in all images ........'
        PRINT,'------------------------------------------------------'

        FOR k = 0, nfiles-1 DO BEGIN

            print,'File : ', files[k]

            info_header = ibis_header(files[k])

            index = WHERE(info_header.filter EQ filter)

            grid = info_header.grid[index]
            stokes = info_header.stokes[index]


            FITS_OPEN, files[k], tmp

            FOR i = 0L, n_elements(index)-1 DO BEGIN

                print, info_header.wave[index[i]]
                FITS_READ, tmp, data, header, exten_no=info_header.extension[index[i]], /NO_PDU
                index_fov  = where(data gt 500.)
                llevel[i,k] = AVG(FLOAT(data[index_fov]))

            ENDFOR

            FITS_CLOSE, tmp

        ENDFOR     

     ENDIF ELSE BEGIN

           files = file_search( dir + fname, COUNT = nfiles)

           nimages = lonarr(nfiles)

           FOR k = 0, nfiles-1 DO nimages[k] = ibis_nimages(files[k])

           ntotal = total(nimages)

           PRINT,'------------------------------------------------------'
           PRINT,'Files found: ', nfiles
           PRINT,'Number of images in individual files: '
           FOR k = 0, nfiles-1 DO print, nimages[k]
           PRINT,'Total number of images: ', ntotal
           PRINT,'------------------------------------------------------'

           llevel = fltarr(max(nimages),nfiles)

           PRINT,'------------------------------------------------------'
           PRINT,'Calculating FOV averaged signal in all images ........'
           PRINT,'------------------------------------------------------'

           FOR k = 0, nfiles-1 DO BEGIN

               print,'File : ', files[k]

               FITS_OPEN, files[k], tmp

               FOR i = 1L, nimages[k] DO BEGIN

                   FITS_READ, tmp, data, header, exten_no= i, /NO_PDU
                   index  = where(data gt 500.)
                   llevel[i-1,k] = AVG(FLOAT(data[index]))

               ENDFOR

               FITS_CLOSE, tmp

           ENDFOR     

     ENDELSE

RETURN, llevel
END
