;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_wavelegth_shift
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION ibis_wavelength_shift, f_file, d_file, npoints, aps, $
                         SAVEFILE = savefile, POL = pol, SINGLE = single, DUAL = dual, $
                         STOKES = stokes

;;***************************************************************
;; Load files
;;***************************************************************

     RESTORE, /VERB, f_file
     RESTORE, /VERB, d_file

;;***************************************************************
;; Get dimensions
;;***************************************************************

     dim   = SIZE(flat)
     Nx    = dim[1]
     Ny    = dim[2]
     IF KEYWORD_SET(STOKES) THEN Npol = stokes ELSE Npol = 1
     Nwave = dim[3]/Npol

;;***************************************************************
;; Get aperture mask
;;***************************************************************

     WINDOW, 0, xs = Nx, ys = Ny
     aps  = ibis_get_aperture(REFORM(flat[*, *, 0]))
     idx  = WHERE(aps GE 1, Num)

     WDELETE

;;***************************************************************
;; Create mean dark, no wavelength dependency assumed
;;***************************************************************

     dark = AVG(dark, 2)
  
;;***************************************************************
;; Order flats when in rabbit mode
;;***************************************************************

     wtmp = REFORM((unique_elements(info_flat_nb.grid)))
     index = sort(wtmp)

     ftmp = flat

     FOR i = 0, Nwave-1 DO ftmp[*,*,i*Npol:(i+1)*Npol-1] = flat[*,*,index[i]*Npol:(index[i]+1)*Npol-1]
 
     flat = ftmp & ftmp = 0

;;---------------------------------------------------------
;;---------------------------------------------------------
;; Distinguish Modes : Spectropolarimetric mode
;;---------------------------------------------------------
;;---------------------------------------------------------

     IF KEYWORD_SET(POL) THEN BEGIN

        print,'------------------------------------------------------'
        print,'Blueshift calculation for spectropolarimetric case ...'
        print,'------------------------------------------------------'

        temp = FLTARR(Nx, Ny)
        flat_tmp = FLTARR(Nx, Ny, Nwave, Npol)

        FOR n =0, Nwave-1 DO BEGIN
            FOR i = 0, Npol-1 DO BEGIN
                temp[idx] = (REFORM(flat[*,*,n*Npol+i] - dark))[idx]
                flat_tmp[*, *, n, i] = temp
            ENDFOR
        ENDFOR

        window, 0, xsize = 700, ysize = 400
        plot, flat_tmp[250,500,*,0], psym=1
        oplot, flat_tmp[250,125,*,0], psym=2
        oplot, flat_tmp[250,900,*,0], psym=4

        PRINT & PAUSE,'Enter wavelength range used for shift calculation (def = use all, enter x y)', /nopause, ans = ans

        wdelete

        IF ans EQ '' THEN BEGIN

	   PRINT,'Processing all'
       
           wrange = [0, Nwave-1]

        ENDIF ELSE BEGIN

              ans = fix(get_words(ans))
              wrange = [ans[0], ans[1]]
   
        ENDELSE
  

        offset = ibis_core_map(flat_tmp[*,*,wrange[0]:wrange[1],*], wrange, $ 
                               npoints, aps, info_flat_nb, POL=pol, DUAL=dual, SINGLE=single, STOKES=stokes)


     ENDIF ELSE BEGIN

;;---------------------------------------------------------
;;---------------------------------------------------------
;; Distinguish Modes : Spectroscopic mode
;;---------------------------------------------------------
;;---------------------------------------------------------

           print,'------------------------------------------------------'
           print,'Blueshift calculation for spectroscopic case ...'
           print,'------------------------------------------------------'

           flat_tmp = FLTARR(Nx, Ny, Nwave)
           temp = FLTARR(Nx, Ny)

           FOR n=0, Nwave-1 DO BEGIN
               temp[idx] = (reform(flat[*,*,n] - dark))[idx]
               flat_tmp[*, *, n] = temp
           ENDFOR

           window, 0, xsize = 700, ysize = 400
           plot, flat_tmp[500,500,*], psym=-1
           oplot, flat_tmp[500,125,*], psym=-2
           oplot, flat_tmp[500,900,*], psym=-4

           PRINT & PAUSE,'Enter wavelength range used for shift calculation (def = use all, enter x y)', /nopause, ans = ans

           wdelete

           IF ans EQ '' THEN BEGIN

	      PRINT,'Processing all'
       
              wrange = [0, n_elements(Nwave)]

           ENDIF ELSE BEGIN

                 ans = fix(get_words(ans))
                 wrange = [ans[0], ans[1]]
   
           ENDELSE
  
           offset = ibis_core_map(flat_tmp[*,*,wrange[0]:wrange[1]], wrange, $ 
                                       npoints, aps, info_flat_nb)

     ENDELSE

;;***************************************************************
;; Save data
;;***************************************************************

     IF KEYWORD_SET(SAVEFILE) THEN BEGIN 

        SAVE, FILENAME = savefile, offset, aps

        PRINT,'------------------------------------------------------'
        PRINT,'Blueshift calculation done                            '
        PRINT,'File saved :',  savefile
        PRINT,'------------------------------------------------------'

     ENDIF

;;***************************************************************
;; Done
;;***************************************************************

     RETURN, offset
END
